#include <cstdlib>
#include <iostream>
#include <vector>

using namespace std;

template <class T> class ArbreGen {
private:
  struct node_arbreGen {
    T info;
    vector<node_arbreGen*> seg;
  };
  node_arbreGen* primer_node;
  

public:

  ArbreGen(){
  /* Pre: cert */
  /* Post: El resultat és un arbre sense cap element */
    primer_node= NULL;
  }
  
  ArbreGen(const T& x){
    /* Pre: cert */
    /* Post: El resultat és un arbre amb x com a arrel i sense fills */
    primer_node= new node_arbreGen;
    primer_node->info = x;
  }

  ArbreGen(const ArbreGen& original)
  /* Pre: cert */
  /* Post: el resultat és una còpia d'original */
  {
    primer_node = copia_node_arbreGen(original.primer_node);
  }


  // Destructora: 

  ~ArbreGen(){
    esborra_node_arbreGen(primer_node);
  }


  ArbreGen& operator=(const ArbreGen& original)
  /* Pre: cert */
  /* Post: El p.i. passa a ser una còpia d'original i qualsevol
     contingut anterior del p.i. ha estat esborrat
     (excepte si el p.i. i original ja eren el mateix objecte) */
  {
    if (this != &original) {
      esborra_node_arbreGen(primer_node);
      primer_node = copia_node_arbreGen(original.primer_node);
    }
    return *this;
  }


// Modificadores


  void a_buit()
  /* Pre: cert */
  /* Post: el p.i. és un arbre buit i qualsevol
     contingut anterior del p.i. ha estat esborrat */
  {
    esborra_node_arbreGen(primer_node);
    primer_node= NULL;
  }

  void plantar(const T &x)
  /* Pre: el p.i. és buit */
  /* Post: el p.i. té x com a arrel i zero fills */
  {
    primer_node= new node_arbreGen;
    primer_node->info= x;
    // No cal fer primer_node->seg = vector<node_arbreGen*>(0);
  } 

  void plantar(const T& x, vector<ArbreGen>& v){
  /* Pre: La longitud d'v és el nombre de fills del paràmetre implícit */
  /* Post: l'arbre té x com a arrel i els n elements de v com
     a fills */
    node_arbreGen* aux= new node_arbreGen;
    aux->info= x;
    int n=v.size();
    aux->seg = vector<node_arbreGen*>(n);
    for (int i=0; i<n; ++i) {
      aux->seg[i] = v[i].primer_node;
      v[i].primer_node = NULL;
    }
    primer_node= aux;
  }

  void afegir_fill(const ArbreGen &a)
  /* Pre: el p.i. i a no són buits */
  /* Post: el p.i. té un fill més que a l'inici, i aquest nou darrer fill
     és una còpia de l'arbre a */
  {
    //(primer_node->seg).push_back(copia_node_arbreGen(a.primer_node));
    (primer_node->seg).push_back(a.primer_node);
    a.primer_node = NULL;
  }


  void fill(const ArbreGen &a, int i)
  /* Pre: el p.i. és buit, a no és buit, i està entre 1 i el nombre de fills d'a */
  /* Post: el p.i és una còpia del fill i-èssim d'a */
  {
    primer_node= copia_node_arbreGen((a.primer_node)->seg[i-1]);
  }


  void fills(vector<ArbreGen> &v)
  /* Pre: el p.i. no és buit i li diem A, v és un vector buit */
  /* Post: el p.i. és buit, v passa a contenir els N fills de lâarbre A */
  {
    node_arbreGen* aux= primer_node;
    int n=aux->seg.size();
    v = vector<ArbreGen> (n);
    for (int i=0; i<n; ++i)
      v[i].primer_node = aux->seg[i];
    primer_node= NULL;
    delete aux;
  }

  //Consultores

  T arrel() const{
  /* Pre: L'arbre no és buit */
  /* Post: El resultat éss l'arrel de l'arbre */
    return primer_node->info;
  }

  bool es_buit() const{
  /* Pre: cert */
  /* Post: El resultat indica si l'arbre és buit o no */
    return (primer_node==NULL);
  }

  int nombre_fills() const{
  /* Pre: cert */
  /* Post: El nombre de fills de l'arbre */
    return (primer_node->seg).size();
  }

private:

  static node_arbreGen* copia_node_arbreGen(node_arbreGen* m){
    node_arbreGen* n;
    if (m==NULL) n=NULL;
    else {
      n = new node_arbreGen;
      n->info = m->info;
      int k = m->seg.size();
      n->seg = vector<node_arbreGen*>(k);
      for (int i=0; i<k; ++i)
	n->seg[i] = copia_node_arbreGen(m->seg[i]);
    }
    return n;
  }


  static void esborra_node_arbreGen(node_arbreGen* m){
    if (m != NULL) {
      for (int i=0; i<m->seg.size(); ++i)
	esborra_node_arbreGen(m->seg[i]);
      delete m;
    }
  }

public:

  int altura(){
    return altura_node(primer_node); 
  }


  void suma(const ArbreGen & a){
    primer_node= suma_node(primer_node,a.primer_node);
  }

  int cami() const {
  /* Pre: cert */
  /* Post: retorna la longitud del camí més llarg d'A */
    return gcami(primer_node).first;
  }


private:

  int altura_node(node_arbreGen* m){
    if (!m) return 0;
    int k=0;  
    for (int i=0; i<m->seg.size(); ++i){
      int a=altura_node(m->seg[i]);
      if (a>k) k=a;
    }
    return k+1;
  }

  node_arbreGen* suma_node(node_arbreGen* n, node_arbreGen* m){
    if (m==NULL) return n;
    if (n==NULL) return copia_node_arbreGen(m);
    n->info += m->info;
    int s=min(n->seg.size(),m->seg.size());
    for (int i=0; i<s; ++i)
      n->seg[i]=suma_node(n->seg[i],m->seg[i]);
    for (int i=s; i<m->seg.size(); ++i)
      n->seg.push_back(copia_node_arbreGen(m->seg[i]));
    return n;
  }

  pair<int,int> gcami(node_arbreGen* n) const {
    if (n==NULL) return make_pair(0,0);
    else{
        int mx=0; //el màxim camí d'entre els fills [0..i-1]
        int ma1=0; //el màxim de les altures dels fills [0..i-1]
        int ma2=0; //el segon màxim de les altures dels fills [0..i-1]
        for (int i=0; i< n->seg.size(); ++i){
          pair<int,int> c=gcami(n->seg[i]);
          if (c.first> mx) mx=c.first;
          if (c.second>ma1) {ma2=ma1; ma1=c.second;}
          else if (c.second>ma2) ma2=c.second;
        }
      return make_pair(max(mx,ma1+ma2+1),
		  ma1+1);
    }
  }


};


void llegir_arbreGen_int(ArbreGen<int>& a, int marca){
/* Pre: a es buit; el canal estandar d'entrada conté el recorregut en preordre
   d'un arbre general A d'enters marcat amb "marca" */  
/* Post: a = A */
  int x;
  cin >> x;
  if (x!= marca) {
    int m;
    cin >> m;
    vector<ArbreGen<int> > v(m);
    for (int i=0; i<m; ++i)
      llegir_arbreGen_int(v[i], marca);
    a.plantar(x,v);
  }
}

void escriure_arbreGen_int(ArbreGen<int> &a) {
/* Pre: cert */ 
/* Post: El canal estandar de sortida cont� el recorregut en preordre d'a */ 
  if (not a.es_buit()) {
    int x = a.arrel();
    int m= a.nombre_fills();
    vector<ArbreGen<int> > va;
    a.fills(va);
    cout << x << " ";
    for (int i=0; i<m; ++i)
      escriure_arbreGen_int(va[i]); 
    a.plantar(x,va);    
  }
}

int main(){

  int n;
  cin >> n;
  for (int i=0; i<n; ++i){
    ArbreGen<int> a;
    llegir_arbreGen_int(a,-1);
    cout <<  a.altura() << " , " << a.cami() << endl;
  }
    ArbreGen<int> a;
    llegir_arbreGen_int(a,-1);
    escriure_arbreGen_int(a);    
    cout << endl;
    ArbreGen<int> b;
    llegir_arbreGen_int(b,-1);
    escriure_arbreGen_int(b);    
    cout << endl;
    a.suma(b);
    escriure_arbreGen_int(a);    
    cout << endl;
}



